package org.mycompany.service.db.client;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.Date;

import javax.jws.WebService;
import javax.xml.namespace.QName;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceClient;

import org.mycompany.service.db.api.MyDBService;
import org.mycompany.service.db.api.MyDBServiceException;
import org.mycompany.service.db.jaxb.Concept;
import org.mycompany.service.db.jaxb.ConceptType;

public class DBClient {

	private static final String MY_DB_SERVICE_URL = "http://localhost:8080/my-manager/my-service";

	public static MyDBService getService() {
		try {
			final WebServiceClient clientAnnotation = MyDBService.class.getAnnotation(WebServiceClient.class);
			final WebService serviceAnnotation = MyDBService.class.getAnnotation(WebService.class);

			return Service.create(new URL(MY_DB_SERVICE_URL),
					new QName(serviceAnnotation.targetNamespace(), clientAnnotation.name())).getPort(
					new QName(serviceAnnotation.targetNamespace(), serviceAnnotation.portName()), MyDBService.class);
		} catch (MalformedURLException e) {
			throw new RuntimeException(e);
		}
	}

	public static void main(String[] args) {
		try {
			System.out.println("Result: " + getService().addConcept("01", "alien", ConceptType.CHILD));
		} catch (MyDBServiceException e) {
			e.printStackTrace();
		}
	}

	/*
	 * This class is not compilable after -Xxew is applied.
	 */
	/*
	public static Concept createDummyConcept_Normal() {
		final Concept concept = new Concept();
		final Concept.Types types = new Concept.Types();

		concept.setCreator("robot");
		concept.setCreationDateTime(new Date());
		concept.setTypes(types);

		types.getType().add(ConceptType.CREATURE);
		types.getType().add(ConceptType.HUMAN);
		types.getType().add(ConceptType.WOMAN);

		return concept;
	}
	 */

	public static Concept createDummyConcept_ListAccess() {
		final Concept concept = new Concept();

		concept.setCreator("computer");
		concept.setCreationDateTime(new Date());

		// -Xxew demo:
		concept.getTypes().add(ConceptType.CREATURE);
		concept.getTypes().add(ConceptType.HUMAN);
		concept.getTypes().add(ConceptType.WOMAN);

		return concept;
	}

	public static Concept createDummyConcept_Construtor() {
		// -Xvalue-constructor demo:
		return new Concept("99", new Date(), "neighbour", Arrays.asList(ConceptType.CREATURE, ConceptType.HUMAN,
				ConceptType.WOMAN));
	}

	public static Concept createDummyConcept_FluentAPI() {
		// -Xfluent-api demo:
		return new Concept().withId("01").withCreationDateTime(new Date()).withCreator("author").withTypes(
				ConceptType.CREATURE, ConceptType.CHILD);
	}
}
